@@ -70,7 +70,7 @@ package EV;
 use strict;
 
 BEGIN {
-   our $VERSION = '1.86';
+   our $VERSION = '2.0';
    use XSLoader;
    XSLoader::load "EV", $VERSION;
 }
@@ -88,6 +88,53 @@ BEGIN {
 @EV::Fork::ISA     =
    "EV::Watcher";
 
+@EV::Loop::Default::ISA = "EV::Loop";
+
+=head1 EVENT LOOPS
+
+EV supports multiple event loops: There is a single "default event loop"
+that can handle everything including signals and child watchers, and any
+number of "dynamic event loops" that can use different backends (with
+various limitations), but no child and signal watchers.
+
+You do not have to do anything to create the default event loop: When
+the module is loaded a suitable backend is selected on the premise of
+selecting a working backend (which for example rules out kqueue on most
+BSDs). Modules should, unless they have "special needs" always use the
+default loop as this is fastest (perl-wise), best supported by other
+modules (e.g. AnyEvent or Coro) and most portable event loop.
+
+For specific programs you can create additional event loops dynamically.
+
+=over 4
+
+=item $loop = new EV::loop [$flags]
+
+Create a new event loop as per the specified flags. Please refer to the
+C<ev_loop_new ()> function description in the libev documentation
+(L<http://pod.tst.eu/http://cvs.schmorp.de/libev/ev.pod#GLOBAL_FUNCTIONS>)
+for more info.
+
+The loop will automatically be destroyed when it is no longer referenced
+by any watcher and the loop object goes out of scope.
+
+Using C<EV::FLAG_FORKCHECK> is recommended, as only the default event loop
+is protected by this module.
+
+=item $loop->loop_fork
+
+Must be called after a fork in the child, before entering or continuing
+the event loop. An alternative is to use C<EV::FLAG_FORKCHECK> which calls
+this fucntion automatically, at some performance loss (refer to the libev
+documentation).
+
+=item $loop = EV::default_loop [$flags]
+
+Return the default loop (which is a singleton object).
+
+=back
+
+
 =head1 BASIC INTERFACE
 
 =over 4
@@ -100,23 +147,43 @@ informative message and continues.
 
 If this callback throws an exception it will be silently ignored.
 
+=item $flags = EV::supported_backends
+
+=item $flags = EV::recommended_backends
+
+=item $flags = EV::embeddable_backends
+
+Returns the set (see C<EV::BACKEND_*> flags) of backends supported by this
+instance of EV, the set of recommended backends (supposed to be good) for
+this platform and the set of embeddable backends (see EMBED WATCHERS).
+
+=item EV::sleep $seconds
+
+Block the process for the given number of (fractional) seconds.
+
 =item $time = EV::time
 
 Returns the current time in (fractional) seconds since the epoch.
 
 =item $time = EV::now
 
+=item $time = $loop->now
+
 Returns the time the last event loop iteration has been started. This
 is the time that (relative) timers are based on, and refering to it is
 usually faster then calling EV::time.
 
-=item $method = EV::method
+=item $backend = EV::backend
+
+=item $backend = $loop->backend
 
 Returns an integer describing the backend used by libev (EV::METHOD_SELECT
 or EV::METHOD_EPOLL).
 
 =item EV::loop [$flags]
 
+=item $loop->loop ([$flags])
+
 Begin checking for events and calling callbacks. It returns when a
 callback calls EV::unloop.
 
@@ -128,6 +195,8 @@ The $flags argument can be one of the following:
 
 =item EV::unloop [$how]
 
+=item $loop->unloop ([$how])
+
 When called with no arguments or an argument of EV::UNLOOP_ONE, makes the
 innermost call to EV::loop return.
 
@@ -136,11 +205,15 @@ fast as possible.
 
 =item $count = EV::loop_count
 
+=item $count = $loop->loop_count
+
 Return the number of times the event loop has polled for new
 events. Sometiems useful as a generation counter.
 
 =item EV::once $fh_or_undef, $events, $timeout, $cb->($revents)
 
+=item $loop->once ($fh_or_undef, $events, $timeout, $cb->($revents))
+
 This function rolls together an I/O and a timer watcher for a single
 one-shot event without the need for managing a watcher object.
 
@@ -155,7 +228,7 @@ timeout. Otherwise a EV::timer with this value will be started.
 
 When an error occurs or either the timeout or I/O watcher triggers, then
 the callback will be called with the received event set (in general
-you can expect it to be a combination of C<EV:ERROR>, C<EV::READ>,
+you can expect it to be a combination of C<EV::ERROR>, C<EV::READ>,
 C<EV::WRITE> and C<EV::TIMEOUT>).
 
 EV::once doesn't return anything: the watchers stay active till either
@@ -164,6 +237,8 @@ invoked.
 
 =item EV::feed_fd_event ($fd, $revents)
 
+=item $loop->feed_fd_event ($fd, $revents)
+
 Feed an event on a file descriptor into EV. EV will react to this call as
 if the readyness notifications specified by C<$revents> (a combination of
 C<EV::READ> and C<EV::WRITE>) happened on the file descriptor C<$fd>.
@@ -173,10 +248,23 @@ C<EV::READ> and C<EV::WRITE>) happened on the file descriptor C<$fd>.
 Feed a signal event into EV. EV will react to this call as if the signal
 specified by C<$signal> had occured.
 
+=item EV::set_io_collect_interval $time
+
+=item $loop->set_io_collect_interval ($time)
+
+=item EV::set_timeout_collect_interval $time
+
+=item $loop->set_timeout_collect_interval ($time)
+
+These advanced functions set the minimum block interval when polling for I/O events and the minimum
+wait interval for timer events. See the libev documentation at
+L<http://pod.tst.eu/http://cvs.schmorp.de/libev/ev.pod#FUNCTIONS_CONTROLLING_THE_EVENT_LOOP> for
+a more detailed discussion.
+
 =back
 
 
-=head2 WATCHER OBJECTS
+=head1 WATCHER OBJECTS
 
 A watcher is an object that gets created to record your interest in some
 event. For instance, if you want to wait for STDIN to become readable, you
@@ -184,7 +272,7 @@ would create an EV::io watcher for that:
 
   my $watcher = EV::io *STDIN, EV::READ, sub {
      my ($watcher, $revents) = @_;
-     warn "yeah, STDIN should not be readable without blocking!\n"
+     warn "yeah, STDIN should now be readable without blocking!\n"
   };
 
 All watchers can be active (waiting for events) or inactive (paused). Only
@@ -278,8 +366,8 @@ the watcher had received the given C<$revents> mask.
 
 =item $revents = $w->clear_pending
 
-If the watcher is pending, this function returns clears its pending status
-and returns its C<$revents> bitset (as if its callback was invoked). If the
+If the watcher is pending, this function clears its pending status and
+returns its C<$revents> bitset (as if its callback was invoked). If the
 watcher isn't pending it does nothing and returns C<0>.
 
 =item $previous_state = $w->keepalive ($bool)
@@ -290,7 +378,7 @@ convinient because it allows you to start your watchers (and your jobs),
 call C<EV::loop> once and when it returns you know that all your jobs are
 finished (or they forgot to register some watchers for their task :).
 
-Sometimes, however, this gets in your way, for example when you the module
+Sometimes, however, this gets in your way, for example when the module
 that calls C<EV::loop> (usually the main program) is not the same module
 as a long-living watcher (for example a DNS client module written by
 somebody else even). Then you might want any outstanding requests to be
@@ -308,12 +396,16 @@ event loop from running just because of that watcher.
 
    my $udp_socket = ...
    my $udp_watcher = EV::io $udp_socket, EV::READ, sub { ... };
-   $udp_watcher->keepalive (0);
+   $1000udp_watcher->keepalive (0);
+
+=item $loop = $w->loop
+
+Return the loop that this watcher is attached to.
 
 =back
 
 
-=head2 WATCHER TYPES
+=head1 WATCHER TYPES
 
 Each of the following subsections describes a single watcher type.
 
@@ -325,6 +417,10 @@ Each of the following subsections describes a single watcher type.
 
 =item $w = EV::io_ns $fileno_or_fh, $eventmask, $callback
 
+=item $w = $loop->io ($fileno_or_fh, $eventmask, $callback)
+
+=item $w = $loop->io_ns ($fileno_or_fh, $eventmask, $callback)
+
 As long as the returned watcher object is alive, call the C<$callback>
 when at least one of events specified in C<$eventmask> occurs.
 
@@ -363,6 +459,10 @@ Returns the previously set event mask and optionally set a new one.
 
 =item $w = EV::timer_ns $after, $repeat, $callback
 
+=item $w = $loop->timer ($after, $repeat, $callback)
+
+=item $w = $loop->timer_ns ($after, $repeat, $callback)
+
 Calls the callback after C<$after> seconds (which may be fractional). If
 C<$repeat> is non-zero, the timer will be restarted (with the $repeat
 value as $after) after the callback returns.
@@ -413,6 +513,10 @@ on the timeout.
 
 =item $w = EV::periodic_ns $at, $interval, $reschedule_cb, $callback
 
+=item $w = $loop->periodic ($at, $interval, $reschedule_cb, $callback)
+
+=item $w = $loop->periodic_ns ($at, $interval, $reschedule_cb, $callback)
+
 Similar to EV::timer, but is not based on relative timeouts but on
 absolute times. Apart from creating "simple" timers that trigger "at" the
 specified time, it can also be used for non-drifting absolute timers and
@@ -546,6 +650,10 @@ optionally set a new one.
 
 =item $w = EV::child_ns $pid, $callback
 
+=item $w = $loop->child ($pid, $callback)
+
+=item $w = $loop->child_ns ($pid, $callback)
+
 Call the callback when a status change for pid C<$pid> (or any pid if
 C<$pid> is 0) has been received. More precisely: when the process receives
 a C<SIGCHLD>, EV will fetch the outstanding exit/wait status for all
@@ -596,6 +704,10 @@ watcher for all pids).
 
 =item $w = EV::stat_ns $path, $interval, $callback
 
+=item $w = $loop->stat ($path, $interval, $callback)
+
+=item $w = $loop->stat_ns ($path, $interval, $callback)
+
 Call the callback when a file status change has been detected on
 C<$path>. The C<$path> does not need to exist, changing from "path exists"
 to "path does not exist" is a status change like any other.
@@ -675,6 +787,10 @@ used to query the actual interval used.
 
 =item $w = EV::idle_ns $callback
 
+=item $w = $loop->idle ($callback)
+
+=item $w = $loop->idle_ns ($callback)
+
 Call the callback when there are no other pending watchers of the same or
 higher priority (excluding check, prepare and other idle watchers of the
 same or lower priority, of course). They are called idle watchers because
@@ -706,6 +822,10 @@ The C<idle_ns> variant doesn't start (activate) the newly created watcher.
 
 =item $w = EV::prepare_ns $callback
 
+=item $w = $loop->prepare ($callback)
+
+=item $w = $loop->prepare_ns ($callback)
+
 Call the callback just before the process would block. You can still
 create/modify any watchers at this point.
 
@@ -724,6 +844,10 @@ The C<prepare_ns> variant doesn't start (activate) the newly created watcher.
 
 =item $w = EV::check_ns $callback
 
+=item $w = $loop->check ($callback)
+
+=item $w = $loop->check_ns ($callback)
+
 Call the callback just after the process wakes up again (after it has
 gathered events), but before any other callbacks have been invoked.
 
@@ -787,6 +911,10 @@ are being called, and only in the child after the fork.
 
 =item $w = EV::fork_ns $callback
 
+=item $w = $loop->fork ($callback)
+
+=item $w = $loop->fork_ns ($callback)
+
 Call the callback before the event loop is resumed in the child process
 after a fork.
 
@@ -795,6 +923,54 @@ The C<fork_ns> variant doesn't start (activate) the newly created watcher.
 =back
 
 
+=head3 EMBED WATCHERS - when one backend isn't enough...
+
+This is a rather advanced watcher type that lets you embed one event loop
+into another (currently only IO events are supported in the embedded
+loop, other types of watchers might be handled in a delayed or incorrect
+fashion and must not be used).
+
+See the libev documentation at
+L<http://pod.tst.eu/http://cvs.schmorp.de/libev/ev.pod#code_ev_embed_code_when_one_backend_>
+for more details.
+
+In short, this watcher is most useful on BSD systems without working
+kqueue to still be able to handle a large number of sockets:
+
+  my $socket_loop;
+
+  # check wether we use SELECT or POLL _and_ KQUEUE is supported
+  if (
+    (EV::backend & (EV::BACKEND_POLL | EV::BACKEND_SELECT))
+    && (EV::supported_backends & EV::embeddable_backends & EV::BACKEND_KQUEUE)
+  ) {
+    # use kqueue for sockets
+    $socket_loop = new EV::Loop EV::BACKEND_KQUEUE | EV::FLAG_NOENV;
+  }
+
+  # use the default loop otherwise
+  $socket_loop ||= EV::default_loop;
+
+=over 4
+
+=item $w = EV::embed $otherloop, $callback
+
+=item $w = EV::embed_ns $otherloop, $callback
+
+=item $w = $loop->embed ($otherloop, $callback)
+
+=item $w = $loop->embed_ns ($otherloop, $callback)
+
+Call the callback when the embedded event loop (C<$otherloop>) has any
+I/O activity. The C<$callback> should alwas be specified as C<undef> in
+this version of EV, which means the embedded event loop will be managed
+automatically.
+
+The C<embed_ns> variant doesn't start (activate) the newly created watcher.
+
+=back
+
+
 =head1 PERL SIGNALS
 
 While Perl signal handling (C<%SIG>) is not affected by EV, the behaviour
@@ -811,8 +987,8 @@ watcher:
 
    my $async_check = EV::check sub { };
 
-This ensures that perl shortly gets into control for a short time, and
-also ensures slower overall operation.
+This ensures that perl gets into control for a short time to handle any
+pending signals, and also ensures (slightly) slower overall operation.
 
 =head1 THREADS
 
@@ -850,8 +1026,9 @@ default_loop
 
 =head1 SEE ALSO
 
-L<EV::ADNS> (asynchronous dns), L<Glib::EV> (makes Glib/Gtk2 use EV as
-event loop), L<Coro::EV> (efficient coroutines with EV).
+L<EV::ADNS> (asynchronous DNS), L<Glib::EV> (makes Glib/Gtk2 use EV as
+event loop), L<EV::Glib> (embed Glib into EV), L<Coro::EV> (efficient
+coroutines with EV), L<Net::SNMP::EV> (asynchronous SNMP).
 
 =head1 AUTHOR
 
